package cd.casic.ci.project.service;




import cd.casic.ci.common.pipeline.pojo.Pagination;
import cd.casic.ci.common.pipeline.pojo.auth.ProjectConditionDTO;
import cd.casic.ci.project.dal.dto.TProjectRecord;
import cd.casic.ci.project.dal.pojo.*;
import cd.casic.ci.project.enums.*;
import cd.casic.ci.project.pojo.*;
import cd.casic.ci.project.pojo.vo.ProjectDiffVO;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import javax.annotation.Nullable;
import javax.security.auth.AuthPermission;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

@SuppressWarnings("ALL")
public interface ProjectService {

    /**
     * 校验项目名称/英文名称是否合法
     */
    void validate(ProjectValidateType validateType, String name, @Nullable String projectId);

    /**
     * 创建项目信息
     */
    String create(
            String userId,
            ProjectCreateInfo projectCreateInfo,
            @Nullable String accessToken,
            ProjectCreateExtInfo createExtInfo,
            @Nullable String defaultProjectId,
            ProjectChannelCode projectChannel
    );

    ProjectVO createExtProject(
            String userId,
            String projectCode,
            ProjectCreateInfo projectCreateInfo,
            boolean needAuth,
            boolean needValidate,
            ProjectChannelCode channel
    );

    /**
     * 根据项目ID/英文ID获取项目信息对象
     * @param englishName projectCode 英文ID
     * @return ProjectVO 如果没有则为null
     */
    ProjectVO getByEnglishName(
            String userId,
            String englishName,
            @Nullable String accessToken
    );

    /**
     * 根据项目ID/英文ID获取项目信息对象
     * @param englishName projectCode 英文ID
     * @return ProjectVO 如果没有则为null
     */
    ProjectVO show(
            String userId,
            String englishName,
            @Nullable String accessToken
    );

    /**
     * 根据项目ID/英文ID获取项目审批中对比信息
     * @param englishName projectCode 英文ID
     * @return ProjectDiffVO 如果没有则为null
     */
    ProjectDiffVO diff(String userId, String englishName, @Nullable String accessToken);

    /**
     * 根据项目ID/英文ID获取项目信息对象
     * @param englishName projectCode 英文ID
     * @return ProjectVO 如果没有则为null
     */
    ProjectVO getByEnglishName(String englishName);

    /**
     * 修改项目信息 englishName是项目英文名，目前平台在api接口上会把他命名成projectId，实际上与t_project表中的project_id字段不同
     * 后续会统一
     */
    boolean update(
            String userId,
            String englishName,
            ProjectUpdateInfo projectUpdateInfo,
            @Nullable String accessToken,
            @Nullable Boolean needApproval
    );

//    /**
//     * 更新Logo TODO FormDataContentDisposition类是 org.glassfish库的，
//                      涉及文件上传模块相关内容先空着后续在看具体解决方案
//     */
//    ProjectLogo updateLogo(
//            String userId,
//            String englishName,
//            InputStream inputStream,
//            FormDataContentDisposition disposition,
//            @Nullable String accessToken
//    );

    /**
     * 上传Logo
     */
    String uploadLogo(
            String userId,
            InputStream inputStream,
            @Nullable String accessToken
    );

    boolean updateProjectName(String userId, String projectId, String projectName);

    /**
     * 获取所有项目信息
     */
    List<ProjectVO> list(
            String userId,
            @Nullable String accessToken,
            @Nullable Boolean enabled,
            boolean unApproved,
            @Nullable ProjectSortType sortType,
            @Nullable ProjectCollation collation
    );

    Pagination<ProjectByConditionDTO> listProjectsForApply(
            String userId,
            @Nullable String accessToken,
            @Nullable String projectName,
            @Nullable String projectId,
            int page,
            int pageSize
    );

    List<ProjectVO> list(String userId);

    List<ProjectVO> list(Set<String> projectCodes, @Nullable Boolean enabled);

    List<ProjectVO> listOnlyByProjectCode(Set<String> projectCodes);

    List<ProjectVO> list(List<String> projectCodes);

    Page<ProjectVO> list(int limit, int offset);

    Page<ProjectVO> listByChannel(int limit, int offset, List<String> projectChannelCode);

    List<ProjectVO> getAllProject();

    List<ProjectByConditionDTO> listProjectsByCondition(
            ProjectConditionDTO projectConditionDTO,
            int limit,
            int offset
    );

    /**
     * 获取用户已的可访问项目列表
     */
    List<ProjectVO> getProjectByUser(String userName);

    HashMap<String, String> getNameByCode(String projectCodes);

    void updateUsableStatus(
            @Nullable String userId,
            String englishName,
            boolean enabled,
            boolean checkPermission
    );

    Page<ProjectVO> searchProjectByProjectName(String projectName, int limit, int offset);

    boolean hasCreatePermission(String userId);

    long getMinId();

    long getMaxId();

    List<ProjectBaseInfo> getProjectListById(long minId, long maxId);

    boolean verifyUserProjectPermission(
            String userId,
            String projectId,
            AuthPermission permission,
            @Nullable String accessToken
    );

    Set<String> listSecrecyProject();

    boolean createProjectUser(String projectId, ProjectCreateUserInfo createInfo);

    boolean relationIamProject(String projectCode, String relationId);

    ProjectVO getProjectByName(String projectName);

    boolean updateProjectProperties(@Nullable String userId, String projectCode, ProjectProperties properties);

    boolean setDisableWhenInactiveFlag(List<String> projectCodes);

    boolean cancelCreateProject(String userId, String projectId);

    boolean cancelUpdateProject(String userId, String projectId);

    boolean isRbacPermission(String projectId);

    boolean updateProjectSubjectScopes(
            String projectId,
            List<SubjectScopeInfo> subjectScopes
    );

    boolean updateProjectCreator(List<ProjectUpdateCreatorDTO> projectUpdateCreatorDtoList);

    List<OperationalProductVO> getOperationalProducts();

    OperationalProductVO getProductByProductId(int productId);

    List<OperationalProductVO> getOperationalProductsByBgName(String bgName);

    void updateProjectProductId(
            String englishName,
            @Nullable String productName,
            @Nullable Integer productId
    );

    void updateOrganizationByEnglishName(
            String englishName,
            ProjectOrganizationInfo projectOrganizationInfo
    );

    ProjectOrganizationInfo fixProjectOrganization(
            TProjectRecord tProjectRecord
    );

    List<ProjectBaseInfo> getProjectListByProductId(
            int productId
    );

    List<String> getExistedEnglishName(
            List<String> englishNameList
    );

    boolean remindUserOfRelatedProduct(
            String userId,
            String englishName
    );

    boolean updatePluginDetailsDisplay(
            String englishName,
            List<PluginDetailsDisplayOrder> pluginDetailsDisplayOrder
    );

    String getPipelineDialect(String projectId);
}
