package cd.casic.pipeline.store.api.common;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.enums.StoreTypeEnum;
import cd.casic.pipeline.store.pojo.common.statistic.StoreDailyStatisticRequest;
import cd.casic.pipeline.store.pojo.common.statistic.StoreStatistic;
import cd.casic.pipeline.store.pojo.common.statistic.StoreStatisticPipelineNumUpdate;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;


@Tag(name = "SERVICE_STORE_STATISTIC", description = "研发商店-统计")
@RestController
@RequestMapping("/service/store/statistic")
public interface ServiceStoreStatisticResource {

    @Operation(summary = "获取store组件统计信息")
    @GetMapping("/types/{storeType}/codes/{storeCode}")
    CommonResult<StoreStatistic> getStatisticByCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode
    );

    @Operation(summary = "更新使用store组件的流水线数量")
    @PutMapping("/types/{storeType}/pipeline/num/update")
    CommonResult<Boolean> updatePipelineNum(
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "使用store组件流水线数量更新实体对象列表", required = true)
            List<StoreStatisticPipelineNumUpdate> pipelineNumUpdateList
    );

    @Operation(summary = "更新store组件的每日统计信息")
    @PutMapping("/types/{storeType}/codes/{storeCode}/daily/info/update")
    CommonResult<Boolean> updateDailyStatisticInfo(
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "store组件的每日统计信息", required = true)
            StoreDailyStatisticRequest storeDailyStatisticRequest
    );
}