package cd.casic.pipeline.store.api.common;


import cd.casic.ci.log.dal.pojo.QueryLogs;
import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.enums.StoreTypeEnum;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;
import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID_DEFAULT_VALUE;

@Tag(name = "USER_STORE_LOG", description = "研发商店-日志")
@RestController
@RequestMapping("/user/store/logs")
public interface UserStoreLogResource {

    @Operation(summary = "根据构建ID获取初始化所有日志")
    @GetMapping("/types/{storeType}/projects/{projectCode}/pipelines/{pipelineId}/builds/{buildId}/")
    CommonResult<QueryLogs> getInitLogs(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "研发商店组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "项目代码", required = true)
            @PathVariable("projectCode")
            String projectCode,
            @Parameter(description = "流水线ID", required = true)
            @PathVariable("pipelineId")
            String pipelineId,
            @Parameter(description = "构建ID", required = true)
            @PathVariable("buildId")
            String buildId,
            @Parameter(description = "是否包含调试日志", required = false)
            @RequestParam("debug")
            Boolean debug,
            @Parameter(description = "对应elementId", required = false)
            @RequestParam("tag")
            String tag,
            @Parameter(description = "执行次数", required = false)
            @RequestParam("executeCount")
            Integer executeCount
    );

    @Operation(summary = "获取更多日志")
    @GetMapping("/types/{storeType}/projects/{projectCode}/pipelines/{pipelineId}/builds/{buildId}/more")
    CommonResult<QueryLogs> getMoreLogs(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "研发商店组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "项目代码", required = true)
            @PathVariable("projectCode")
            String projectCode,
            @Parameter(description = "流水线ID", required = true)
            @PathVariable("pipelineId")
            String pipelineId,
            @Parameter(description = "构建ID", required = true)
            @PathVariable("buildId")
            String buildId,
            @Parameter(description = "是否包含调试日志", required = false)
            @RequestParam("debug")
            Boolean debug,
            @Parameter(description = "日志行数", required = false)
            @RequestParam("num")
            Integer num,
            @Parameter(description = "是否正序输出", required = false)
            @RequestParam("fromStart")
            Boolean fromStart,
            @Parameter(description = "起始行号", required = true)
            @RequestParam("start")
            Long start,
            @Parameter(description = "结尾行号", required = true)
            @RequestParam("end")
            Long end,
            @Parameter(description = "对应elementId", required = false)
            @RequestParam("tag")
            String tag,
            @Parameter(description = "执行次数", required = false)
            @RequestParam("executeCount")
            Integer executeCount
    );

    @Operation(summary = "获取某行后的日志")
    @GetMapping("/types/{storeType}/projects/{projectCode}/pipelines/{pipelineId}/builds/{buildId}/after")
    CommonResult<QueryLogs> getAfterLogs(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "研发商店组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "项目代码", required = true)
            @PathVariable("projectCode")
            String projectCode,
            @Parameter(description = "流水线ID", required = true)
            @PathVariable("pipelineId")
            String pipelineId,
            @Parameter(description = "构建ID", required = true)
            @PathVariable("buildId")
            String buildId,
            @Parameter(description = "起始行号", required = true)
            @RequestParam("start")
            Long start,
            @Parameter(description = "是否包含调试日志", required = false)
            @RequestParam("debug")
            Boolean debug,
            @Parameter(description = "对应elementId", required = false)
            @RequestParam("tag")
            String tag,
            @Parameter(description = "执行次数", required = false)
            @RequestParam("executeCount")
            Integer executeCount
    );
}